#!/usr/bin/perl -w

# OTU 名は一致している必要はありません．OTU の順番は一致させてください．
# cDNA インファイルからはギャプを取り除いておいてください．

# インファイルのギャップを取り除いた状態でアミノ酸と DNA で長さを比較してスクリーンアウトしています (あまり必要ないような気がしてきました)．
# 長さが異なることがよくありますが，これは「A-T」などはアミノ酸配列では X などになっているためだと思われます．

# UTRs やポリ A が入っている場合は，一度 pal2nal で処理してこれらを取り除く必要があります．

use strict;
use warnings;

my $infileAA  = "AAincGap.fas";
my $infileDNA = "DNAnoGap.fas";

my $outFile   = "outDNAgapped.fas";

&gapAddCDNA($infileAA, $infileDNA, $outFile);

exit;

sub gapAddCDNA {
  my($infileAASR, $infileDNASR, $outFileSR) = @_;
  my $name = "";
  my $seq = "";
  
  my @inFileContAA = ( );
  my @inFileContDNA = ( );
  
  my @nameLineAA = ( );
  my @nameLineDNA = ( );
  
  my %fasHashAA = ( );          # インファイルの AA 配列
  my %fasHashAAnoGap = ( );     # ギャップを削除した AA 配列．
  my %fasHashDNA = ( );         # インファイルの DNA 配列
  my %fasHashDNAnoGap = ( );    # ギャップを削除した DNA 配列
  my %fasHashGappedDNA = ( );   # アミノ酸アライメントに合わせて得られた DNA 配列．
  
  
  open(INAA,"$infileAASR") or die "$!";
  @inFileContAA = <INAA>;
  close INAA;
  
  open(INDNA,"$infileDNASR") or die "$!";
  @inFileContDNA = <INDNA>;
  close INDNA;
  
  
  
  my ($ref_fasHashAA,$ref_nameArrayAA) = &fasHashReader(\@inFileContAA);
  @nameLineAA = @$ref_nameArrayAA;
  %fasHashAA = %$ref_fasHashAA;
  foreach (keys %fasHashAA) {
    my $aaSeqNoGap = $fasHashAA{$_};
    $aaSeqNoGap =~ s/-//g;
    $aaSeqNoGap =~ s/ //g;
    $fasHashAAnoGap{$_} = $aaSeqNoGap;
  }
  
  my ($ref_fasHashDNA,$ref_nameArrayDNA) = &fasHashReader(\@inFileContDNA);
  @nameLineDNA = @$ref_nameArrayDNA;
  %fasHashDNA = %$ref_fasHashDNA;
  foreach (keys %fasHashDNA) {
    my $dnaSeqNoGap = $fasHashDNA{$_};
    $dnaSeqNoGap =~ s/ //g;
    $dnaSeqNoGap =~ s/-//g;
    $fasHashDNAnoGap{$_} = $dnaSeqNoGap;
  }
  
  print "Number of OTUs: AA ".@nameLineAA.", DNA ".@nameLineDNA."\n\n";
  for (my $i=0; $i < scalar@nameLineAA; $i++) {
    print "Length without gap, AA:$nameLineAA[$i] ".length($fasHashAAnoGap{$nameLineAA[$i]})."aa ";
    print "DNA: $nameLineDNA[$i] ".length($fasHashDNAnoGap{$nameLineDNA[$i]})."bp\n";
  }
  
  
  my $aaSeq  = "";
  my $dnaSeq = "";
  
  for (my $j=0; $j < scalar@nameLineAA; $j++){
    my $stringAA = $fasHashAA{$nameLineAA[$j]};
    my $stringDNA = $fasHashDNA{$nameLineDNA[$j]};
    my $nagasaAA = length$stringAA;
      
  #  print "stringAA ".$stringAA."\n";
    
    for (my $p=0; $p < $nagasaAA; $p++) {
      my $residue = "";
      my $triplet = "";
  
      $stringAA =~ s/^(.)//;
      $residue = $1;
      
      if ($residue eq "-") {
        $triplet = "---";
      } else {
        $stringDNA =~ s/(...)//;
        $triplet = $1;
      }
  
      $aaSeq  .= $residue;
      $dnaSeq .= $triplet;    
  
    }
    
  
    ##### サイト番号付きで確認したいときは，これをスクリーンアウトする．####
  #  print "$nameLineAA[$j]\n";
  #  print "$aaSeq\n";
  #  print "$dnaSeq\n";
  #  print "\n\n";
  
    $fasHashGappedDNA{$nameLineDNA[$j]} = $dnaSeq;
  
    $aaSeq = "";
    $dnaSeq = "";
  
  }
  
  
  open(OUT,">$outFileSR");
  foreach (@nameLineDNA){
    print OUT "$_\n$fasHashGappedDNA{$_}\n";
  }
  close OUT;

}


sub fasHashReader
{
my $name = "";
my $seq = "";
my $stone = 0;
my @nameArraySR = ( );
my %fasHashSR = ( );

my ($infileDNAContSR) = @_;

foreach my $infileDNALine (@$infileDNAContSR) {
  chomp($infileDNALine);
  if($infileDNALine =~ /^>/){  
    if($stone == 0){$name = $infileDNALine;
      $stone = 1;  
    } else {
      $seq =~ s/ //g;
      $fasHashSR{$name} = $seq;  
      push(@nameArraySR,$name);
      $name = "$infileDNALine";
      $seq = "";
    }
  } else {
    $seq .= $infileDNALine; 
  }
}
$seq =~ s/ //g;
$fasHashSR{$name} = $seq; 
push(@nameArraySR,$name);

return (\%fasHashSR,\@nameArraySR);

}